#ifndef SELECTIONTOOLS_H
#define SELECTIONTOOLS_H

//
// You should use this namespace to define general object selection
//

#include <vector>
#include <iostream>
#include <cmath>

#include "BlockClasses.h"

namespace CppTools
{

  // Structures used to define object selection cuts

  //! Electron definition cuts
  struct ElectronCuts {
    //! Object definition
    float ptMin;
    float etaMax;
    bool useEtCone;
    float etConeMax;
    //! Overlap definition
    float deltaRMax_ej;
    float deltaRVeto_ej;
    //! General flag for Atlfast1 use
    bool applyOnlyAcceptanceCuts;
    bool applyOverlapRemoval;
    //! Static function defining standard susy cuts
    static ElectronCuts StandardElectronCuts();
    //! ROOT Dict ClassDef declaration
    ClassDef(ElectronCuts, 1)
  };


  //! Photon definition cuts
  struct PhotonCuts {
    //! Object definition
    float ptMin;
    float etaMax;
    bool useEtCone;
    float etConeMax;
    //! Overlap definition
    float deltaRMax_gammaj;
    float deltaRVeto_gammaj;
    //! General flag for Atlfast1 use
    bool applyOnlyAcceptanceCuts;
    bool applyOverlapRemoval;
    //! Static function defining standard susy cuts
    static PhotonCuts StandardPhotonCuts();
    //! ROOT Dict ClassDef declaration
    ClassDef(PhotonCuts, 1)
  };


  //! Muon definition cuts
  struct MuonCuts {
    //! Object definition
    float ptMin;
    float etaMax;
    float matchChi2Max;
    bool useEtCone;
    float etConeMax;
    //! Overlap definition
    float deltaRVeto_mj;
    //! General flag for Atlfast1 use
    bool applyOnlyAcceptanceCuts;
    bool applyOverlapRemoval;
    //! Static function defining standard susy cuts
    static MuonCuts StandardMuonCuts();
    //! ROOT Dict ClassDef declaration
    ClassDef(MuonCuts, 1)
  };

  //! Jet definition cuts
  struct JetCuts {
    //! Object definition
    float ptMin;
    float etaMax;
    //! Static function defining standard susy cuts
    static JetCuts StandardJetCuts();
    //! ROOT Dict ClassDef declaration
    ClassDef(JetCuts, 1)

  };

  //! SpclMc container cuts
  struct TruthCuts {
    static TruthCuts StandardTruthCuts();
    //! ROOT Dict ClassDef declaration
    ClassDef(TruthCuts, 1)
  };

  //! Tau definition cuts
  struct TauCuts {
    //! Object definition
    float ptMin;
    float etaMax;
    float llhMin;
    //! Overlap definition
    float deltaRMax_tj;
    float deltaRMax_te;
    float deltaRMax_tm;
    //! General flag for Atlfast1 use
    bool applyOnlyAcceptanceCuts;
    bool applyOverlapRemoval;
    //! Flag to select only tauRec taus
    bool SelectTauRecOnly;
    //! Static function defining standard susy cuts
    static TauCuts StandardTauCuts();
    //! ROOT Dict ClassDef declaration
    ClassDef(TauCuts, 1)
  };

  // General functions

  //! Electron selection function
  bool IsSelected(const TRootElectron& electron,
		  const CppTools::ElectronCuts& electronCuts = CppTools::ElectronCuts::StandardElectronCuts());

  //! Photon selection function
  bool IsSelected(const TRootPhoton& photon,
		  const CppTools::PhotonCuts& photonCuts = CppTools::PhotonCuts::StandardPhotonCuts());

  //! Muon selection function
  bool IsSelected(const TRootMuon& muon,
		  const CppTools::MuonCuts& muonCuts = CppTools::MuonCuts::StandardMuonCuts());

  //! Jet selection function
  bool IsSelected(const TRootJet& jet,
		  const CppTools::JetCuts& jetCuts = CppTools::JetCuts::StandardJetCuts());

//  //! Truth selection function
//  bool IsSelected(const TruthParticle& particle,
//		  const CppTools::TruthCuts& truthCuts = CppTools::TruthCuts::StandardTruthCuts());

  //! Tau selection function
  bool IsSelected(const TRootTauJet& tau,
		  const CppTools::TauCuts& tauCuts = CppTools::TauCuts::StandardTauCuts());

  //! Overlap removal function between jet and electron (remove jet)
  bool IsOverlapping(const TRootJet& jet, const TRootElectron& electron,
		     const CppTools::ElectronCuts& electronCuts = CppTools::ElectronCuts::StandardElectronCuts());

  //! Overlap removal function between jet and photon (remove jet)
  bool IsOverlapping(const TRootJet& jet, const TRootPhoton& photon,
		     const CppTools::PhotonCuts& photonCuts = CppTools::PhotonCuts::StandardPhotonCuts());

  //! Overlap removal function between jet and tau (remove jet)
  bool IsOverlapping(const TRootJet& jet, const TRootTauJet& tau,
		     const CppTools::TauCuts& tauCuts = CppTools::TauCuts::StandardTauCuts());

  //! Overlap removal function between tau and electron (remove tau)
  bool IsOverlapping(const TRootElectron& electron, const TRootTauJet& tau,
		     const CppTools::TauCuts& tauCuts = CppTools::TauCuts::StandardTauCuts());

  //! Overlap removal function between muon and tau (remove muon)
  bool IsOverlapping(const TRootMuon& muon, const TRootTauJet& tau,
		     const CppTools::TauCuts& tauCuts = CppTools::TauCuts::StandardTauCuts());

  //! Veto function between electron and jet (remove electron)
  bool IsVetoed(const TRootElectron& electron, const TRootJet& jet,
		const CppTools::ElectronCuts& electronCuts = CppTools::ElectronCuts::StandardElectronCuts());

  //! Veto function between muon and jet (remove muon)
  bool IsVetoed(const TRootMuon& muon, const TRootJet& jet,
		const CppTools::MuonCuts& muonCuts = CppTools::MuonCuts::StandardMuonCuts());

  //! Veto function between photon and jet (remove photon)
  bool IsVetoed(const TRootPhoton& photon, const TRootJet& jet,
		const CppTools::PhotonCuts& photonCuts = CppTools::PhotonCuts::StandardPhotonCuts());

//  //! Get closest electron from a jet
//  const TRootElectron* GetClosestElectron(ElectronContainer * pElCont, const TRootJet& jet,
//					       const CppTools::ElectronCuts& electronCuts = CppTools::ElectronCuts::StandardElectronCuts());
//
//  //! Get closest photon from a jet
//  const TRootPhoton* GetClosestPhoton(PhotonContainer * pPhCont, const TRootJet& jet,
//					   const CppTools::PhotonCuts& photonCuts = CppTools::PhotonCuts::StandardPhotonCuts());
//
//  //! Get closest muon from a jet
//  const TRootMuon* GetClosestMuon(Analysis::MuonContainer * pMuCont, const TRootJet& jet,
//				       const CppTools::MuonCuts& muonCuts = CppTools::MuonCuts::StandardMuonCuts());
//
//  //! Compute deltaR from closest electron to a jet
//  double DeltaRClosestElectron(ElectronContainer * pElCont, const TRootJet& jet,
//			       const CppTools::ElectronCuts& electronCuts = CppTools::ElectronCuts::StandardElectronCuts());
//
//  //! Compute deltaR from closest photon to a jet
//  double DeltaRClosestPhoton(PhotonContainer * pPhCont, const TRootJet& jet,
//			     const CppTools::PhotonCuts& photonCuts = CppTools::PhotonCuts::StandardPhotonCuts());
//
//  //! Compute deltaR from closest muon to a jet
//  double DeltaRClosestMuon(Analysis::MuonContainer * pMuCont, const TRootJet& jet,
//			   const CppTools::MuonCuts& muonCuts = CppTools::MuonCuts::StandardMuonCuts());

//--------------------

//! To get TLorentzVector from Delphes objects
const TLorentzVector GetFourVector(const TRootParticle& p);


} //end of CppTools namespace

#endif // SELECTIONTOOLS_H
